(function (global, factory) {
	if (typeof define === "function" && define.amd) {
		define(['exports', 'react', 'prop-types', './styles/index.css'], factory);
	} else if (typeof exports !== "undefined") {
		factory(exports, require('react'), require('prop-types'), require('./styles/index.css'));
	} else {
		var mod = {
			exports: {}
		};
		factory(mod.exports, global.react, global.propTypes, global.index);
		global.index = mod.exports;
	}
})(this, function (exports, _react, _propTypes) {
	'use strict';

	Object.defineProperty(exports, "__esModule", {
		value: true
	});

	var _react2 = _interopRequireDefault(_react);

	var _propTypes2 = _interopRequireDefault(_propTypes);

	function _interopRequireDefault(obj) {
		return obj && obj.__esModule ? obj : {
			default: obj
		};
	}

	var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) {
		return typeof obj;
	} : function (obj) {
		return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;
	};

	function _classCallCheck(instance, Constructor) {
		if (!(instance instanceof Constructor)) {
			throw new TypeError("Cannot call a class as a function");
		}
	}

	var _createClass = function () {
		function defineProperties(target, props) {
			for (var i = 0; i < props.length; i++) {
				var descriptor = props[i];
				descriptor.enumerable = descriptor.enumerable || false;
				descriptor.configurable = true;
				if ("value" in descriptor) descriptor.writable = true;
				Object.defineProperty(target, descriptor.key, descriptor);
			}
		}

		return function (Constructor, protoProps, staticProps) {
			if (protoProps) defineProperties(Constructor.prototype, protoProps);
			if (staticProps) defineProperties(Constructor, staticProps);
			return Constructor;
		};
	}();

	function _possibleConstructorReturn(self, call) {
		if (!self) {
			throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
		}

		return call && (typeof call === "object" || typeof call === "function") ? call : self;
	}

	function _inherits(subClass, superClass) {
		if (typeof superClass !== "function" && superClass !== null) {
			throw new TypeError("Super expression must either be null or a function, not " + typeof superClass);
		}

		subClass.prototype = Object.create(superClass && superClass.prototype, {
			constructor: {
				value: subClass,
				enumerable: false,
				writable: true,
				configurable: true
			}
		});
		if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass;
	}

	var Dropdown = function (_Component) {
		_inherits(Dropdown, _Component);

		function Dropdown(props) {
			_classCallCheck(this, Dropdown);

			var _this = _possibleConstructorReturn(this, (Dropdown.__proto__ || Object.getPrototypeOf(Dropdown)).call(this, props));

			_this.hide = _this.hide.bind(_this);
			_this.state = {
				visible: false,
				selected: null,
				searchStr: null
			};
			return _this;
		}

		_createClass(Dropdown, [{
			key: 'select',
			value: function select(item) {
				if (this.props.type === 'search') {
					this.setState({ selected: item, visible: false });
				} else {
					this.setState({ selected: item });
				}

				this.props.selected(item);
			}
		}, {
			key: 'show',
			value: function show() {
				this.setState({ visible: true });
				document.addEventListener('click', this.hide);
			}
		}, {
			key: 'hide',
			value: function hide() {
				this.setState({ visible: false });
				document.removeEventListener('click', this.hide);
			}
		}, {
			key: 'showWithoutEventListener',
			value: function showWithoutEventListener() {
				this.setState({ visible: true });
			}
		}, {
			key: 'search',
			value: function search(e) {
				this.setState({ searchStr: e.target.value });
			}
		}, {
			key: 'renderListItems',
			value: function renderListItems() {
				var self = this;
				var _props = this.props,
				    options = _props.options,
				    list = _props.list;

				var items = [];

				if (this.state.searchStr) {
					var _loop = function _loop(i) {
						var item = list[i];

						if (item.text.toLowerCase().indexOf(self.state.searchStr) > -1) {
							items.push(_react2.default.createElement(
								'div',
								{
									className: 'dropdown__listItem',
									key: item.text,
									onClick: function onClick() {
										self.select(item);
									}
								},
								item.text
							));
						}
					};

					for (var i = 0; i < list.length; i++) {
						_loop(i);
					}

					return items;
				}

				var _loop2 = function _loop2(i) {
					var item = list[i];

					if (options && options.formatResult) {
						var newItem = options.formatResult(item, self.state.selected, self.select.bind(self));
						items.push(newItem);
					} else if (item === self.state.selected) {
						items.push(_react2.default.createElement(
							'div',
							{
								className: 'dropdown__listItem--selected',
								key: item.text,
								onClick: function onClick() {
									self.select(item);
								}
							},
							item.text,
							_react2.default.createElement('i', { className: 'fa fa-check dropdown__selectedItem' })
						));
					} else {
						items.push(_react2.default.createElement(
							'div',
							{
								className: 'dropdown__listItem',
								key: item.text,
								onClick: function onClick() {
									self.select(item);
								}
							},
							item.text
						));
					}
				};

				for (var i = 0; i < list.length; i++) {
					_loop2(i);
				}

				return items;
			}
		}, {
			key: 'formatSelected',
			value: function formatSelected() {
				var _props2 = this.props,
				    list = _props2.list,
				    placeholder = _props2.placeholder,
				    options = _props2.options,
				    icon = _props2.icon;


				if (this.state.selected) {
					if (options && options.formatSelect) {
						var ret = options.formatSelect(this.state.selected);
						if ((typeof ret === 'undefined' ? 'undefined' : _typeof(ret)) === 'object' && _typeof(ret.props) === 'object') {
							return { text: ret };
						}

						return ret;
					}

					return { text: this.state.selected.text };
				}

				if (this.props.placeholder) {
					return { text: this.props.placeholder };
				}

				if (this.props.list.length) {
					return { text: this.props.list[0].text };
				}

				return { text: 'Not set' };
			}
		}, {
			key: 'render',
			value: function render() {
				var _this2 = this;

				var _props3 = this.props,
				    icon = _props3.icon,
				    width = _props3.width,
				    height = _props3.height,
				    type = _props3.type,
				    placeholder = _props3.placeholder,
				    color = _props3.color;

				var res = this.formatSelected();
				var dropdownHead = void 0;

				dropdownHead = _react2.default.createElement(
					'div',
					{
						className: 'dropdown__head' + (this.state.visible ? ' dropdown__head--clicked' : ''),
						onClick: function onClick() {
							type === 'search' ? _this2.showWithoutEventListener() : _this2.show();
						}
					},
					icon && _react2.default.createElement('i', { className: icon }),
					_react2.default.createElement(
						'span',
						{ className: icon ? 'dropdown__title--icon' : 'dropdown__title' },
						res.text
					),
					_react2.default.createElement('i', { className: 'fa fa-angle-down' })
				);

				if (res.image) {
					dropdownHead = _react2.default.createElement(
						'div',
						{
							className: 'dropdown__head' + (this.state.visible ? ' dropdown__head--clicked' : ''),
							onClick: function onClick() {
								type === 'search' ? _this2.showWithoutEventListener() : _this2.show();
							}
						},
						_react2.default.createElement('img', { className: 'dropdown__avatar', src: res.image }),
						_react2.default.createElement(
							'span',
							{ className: 'dropdown__title--icon' },
							res.text
						),
						_react2.default.createElement('i', { className: 'fa fa-angle-down' })
					);
				}

				if (type === 'search' && this.state.visible) {
					dropdownHead = _react2.default.createElement(
						'div',
						{ className: 'dropdown__head' + (this.state.visible ? ' dropdown__head--clicked' : '') },
						_react2.default.createElement('input', {
							className: 'dropdown__search',
							placeholder: placeholder ? placeholder : 'Search',
							type: 'text',
							defaultValue: this.state.searchStr,
							onChange: this.search.bind(this)
						}),
						_react2.default.createElement('i', {
							className: 'fa fa-search',
							title: 'Click this icon to close the dropdown.',
							onClick: function onClick() {
								_this2.hide();
							}
						})
					);
				}

				return _react2.default.createElement(
					'div',
					{
						style: { width: width ? '' + width : null },
						className: 'dropdown' + (this.state.visible ? ' dropdown--show' : '') + (color ? ' ' + color : '')
					},
					dropdownHead,
					_react2.default.createElement(
						'div',
						{ className: 'dropdown__body' },
						_react2.default.createElement(
							'div',
							{ className: 'dropdown__bodyHolder', style: { maxHeight: height ? height : '174px' } },
							this.renderListItems()
						)
					)
				);
			}
		}]);

		return Dropdown;
	}(_react.Component);

	Dropdown.propTypes = {
		list: _propTypes2.default.array,
		selected: _propTypes2.default.func
	};

	exports.default = Dropdown;
});